@@ -15,21 +15,24 @@ from rest_framework.generics import GenericAPIView
 from rest_framework.schemas.inspectors import ViewInspector
 from rest_framework.schemas.utils import get_pk_description
 from rest_framework.settings import api_settings
+from rest_framework.utils.model_meta import get_field_info
 from rest_framework.views import APIView
 
-from drf_spectacular.fields import OpenApiSerializerFieldExtension
+from drf_spectacular.extensions import (
+    OpenApiSerializerFieldExtension, OpenApiSerializerExtension
+)
 from drf_spectacular.settings import spectacular_settings
 from drf_spectacular.contrib.authentication import *  # noqa: F403, F401
 from drf_spectacular.contrib.serializers import *  # noqa: F403, F401
 from drf_spectacular.plumbing import (
     build_basic_type, warn, anyisinstance, force_instance, is_serializer,
     follow_field_source, is_field, is_basic_type, build_array_type,
-    ComponentRegistry, ResolvedComponent, build_parameter_type
+    ComponentRegistry, ResolvedComponent, build_parameter_type, error,
+    resolve_regex_path_parameter, safe_ref, has_override, get_override,
 )
 from drf_spectacular.types import OpenApiTypes
 from drf_spectacular.utils import OpenApiParameter
 from drf_spectacular.authentication import OpenApiAuthenticationExtension
-from drf_spectacular.serializers import OpenApiSerializerExtension
 
 
 class AutoSchema(ViewInspector):
@@ -41,9 +44,10 @@ class AutoSchema(ViewInspector):
         'delete': 'destroy',
     }
 
-    def get_operation(self, path, method, registry: ComponentRegistry):
+    def get_operation(self, path, path_regex, method, registry: ComponentRegistry):
         self.registry = registry
         self.path = path
+        self.path_regex = path_regex
         self.method = method
 
         operation = {}
@@ -256,11 +260,16 @@ class AutoSchema(ViewInspector):
         model = getattr(getattr(self.view, 'queryset', None), 'model', None)
         parameters = []
 
-        for variable in variables:
+        for idx, variable in enumerate(variables):
             schema = build_basic_type(OpenApiTypes.STR)
             description = ''
+            required = True
 
-            if not model:
+            resolved_parameter = resolve_regex_path_parameter(self.path_regex, variable)
+
+            if resolved_parameter:
+                schema, required = resolved_parameter['schema'], resolved_parameter['required']
+            elif not model:
                 warn(
                     f'could not derive type of path parameter "{variable}" because '
                     f'{self.view.__class__} has no queryset. consider annotating the '
@@ -284,7 +293,7 @@ class AutoSchema(ViewInspector):
             parameters.append({
                 "name": variable,
                 "in": "path",
-                "required": True,
+                "required": required,
                 "description": description,
                 'schema': schema,
             })
@@ -352,29 +361,46 @@ class AutoSchema(ViewInspector):
             mapping['type'] = type
         return mapping
 
-    def _map_model_field(self, field):
-        assert isinstance(field, models.Field)
-        drf_mapping = serializers.ModelSerializer.serializer_field_mapping
+    def _map_model_field(self, model_field):
+        assert isinstance(model_field, models.Field)
+        # to get a fully initialized serializer field we use DRF's own init logic
+        try:
+            field_cls, field_kwargs = serializers.ModelSerializer().build_field(
+                field_name=model_field.name,
+                info=get_field_info(model_field.model),
+                model_class=model_field.model,
+                nested_depth=0,
+            )
+            field = field_cls(**field_kwargs)
+        except:  # noqa
+            field = None
 
-        if field.__class__ in drf_mapping:
-            # use DRF native field resolution - taken from ModelSerializer.get_fields()
-            return self._map_serializer_field(drf_mapping[field.__class__]())
-        elif isinstance(field, models.ForeignKey):
-            return self._map_model_field(field.target_field)
+        # For some cases, the DRF init logic either breaks (custom field with internal type) or
+        # the resulting field is underspecified with regards to the schema (ReadOnlyField).
+        if field and not anyisinstance(field, [serializers.ReadOnlyField, serializers.ModelField]):
+            return self._map_serializer_field(field)
+        elif isinstance(model_field, models.ForeignKey):
+            return self._map_model_field(model_field.target_field)
+        elif hasattr(models, model_field.get_internal_type()):
+            # be graceful when the model field is not explicitly mapped to a serializer
+            internal_type = getattr(models, model_field.get_internal_type())
+            field_cls = serializers.ModelSerializer.serializer_field_mapping[internal_type]
+            return self._map_serializer_field(field_cls())
         else:
-            warn(
-                f'could not resolve model field "{field}" due to missing mapping.'
-                'either your field is custom and not based on a known subclasses '
-                'or we missed something. let us know.'
+            error(
+                f'could not resolve model field "{model_field}". failed to resolve through '
+                f'serializer_field_mapping, get_internal_type(), or any override mechanism. '
+                f'defaulting to "string"'
             )
             return build_basic_type(OpenApiTypes.STR)
 
     def _map_serializer_field(self, field):
-        if hasattr(field, '_spectacular_annotation'):
-            if is_basic_type(field._spectacular_annotation):
-                return build_basic_type(field._spectacular_annotation)
+        if has_override(field, 'field'):
+            override = get_override(field, 'field')
+            if is_basic_type(override):
+                return build_basic_type(override)
             else:
-                return self._map_serializer_field(field._spectacular_annotation)
+                return self._map_serializer_field(override)
 
         serializer_field_extension = OpenApiSerializerFieldExtension.get_match(field)
         if serializer_field_extension:
@@ -520,6 +546,11 @@ class AutoSchema(ViewInspector):
             elif isinstance(target, models.Field):
                 return self._map_model_field(target)
 
+        # DRF was not able to match the model field to an explicit SerializerField and therefore
+        # used its generic fallback serializer field that simply wraps the model field.
+        if isinstance(field, serializers.ModelField):
+            return self._map_model_field(field.model_field)
+
         warn(f'could not resolve serializer field {field}. defaulting to "string"')
         return build_basic_type(OpenApiTypes.STR)
 
@@ -539,6 +570,7 @@ class AutoSchema(ViewInspector):
             return self._map_basic_serializer(serializer, direction)
 
     def _map_basic_serializer(self, serializer, direction):
+        serializer = force_instance(serializer)
         required = []
         properties = {}
 
@@ -558,17 +590,15 @@ class AutoSchema(ViewInspector):
             if field.allow_null:
                 schema['nullable'] = True
             if field.default is not None and field.default != empty and not callable(field.default):
-                schema['default'] = field.to_representation(field.default)
+                default = field.to_representation(field.default)
+                if isinstance(default, set):
+                    default = list(default)
+                schema['default'] = default
             if field.help_text:
                 schema['description'] = str(field.help_text)
             self._map_field_validators(field, schema)
 
-            # sibling entries to $ref will be ignored as it replaces itself and its context with
-            # the referenced object. Wrap it in a separate context.
-            if '$ref' in schema and len(schema) > 1:
-                schema = {'allOf': [{'$ref': schema.pop('$ref')}], **schema}
-
-            properties[field.field_name] = schema
+            properties[field.field_name] = safe_ref(schema)
 
         result = {
             'type': 'object',
@@ -613,7 +643,7 @@ class AutoSchema(ViewInspector):
                     schema['minimum'] = -schema['maximum']
 
     def _map_type_hint(self, method):
-        hint = getattr(method, '_spectacular_annotation', None) or typing.get_type_hints(method).get('return')
+        hint = get_override(method, 'field') or typing.get_type_hints(method).get('return')
 
         if is_serializer(hint) or is_field(hint):
             return self._map_serializer_field(force_instance(hint))
@@ -668,15 +698,15 @@ class AutoSchema(ViewInspector):
                 elif hasattr(view, 'serializer_class'):
                     return view.serializer_class
                 else:
-                    warn(
+                    error(
                         f'Unable to guess serializer for {view.__class__.__name__}. This is graceful '
                         f'fallback handling for APIViews. Consider using GenericAPIView as view base '
                         f'class, if view is under your control. ignoring view for now. '
                     )
             else:
-                warn('Encountered unknown view base class. please report this issue. ignoring for now')
+                error('Encountered unknown view base class. please report this issue. ignoring for now')
         except Exception as exc:
-            warn(
+            error(
                 f'Exception raised while getting serializer from {view.__class__.__name__}. Hint: '
                 f'Is get_serializer_class() returning None or is get_queryset() not working without '
                 f'a request? Ignoring the view for now. (Exception: {exc})'
@@ -776,7 +806,7 @@ class AutoSchema(ViewInspector):
             schema = build_basic_type(OpenApiTypes.OBJECT)
             schema['description'] = 'Unspecified response body'
 
-        if self._is_list_view(serializer):
+        if self._is_list_view(serializer) and not get_override(serializer, 'many') is False:
             schema = build_array_type(schema)
             paginator = self._get_paginator()
             if paginator:
@@ -795,12 +825,18 @@ class AutoSchema(ViewInspector):
     def _get_serializer_name(self, serializer, direction):
         serializer_extension = OpenApiSerializerExtension.get_match(serializer)
         if serializer_extension and serializer_extension.get_name():
-            return serializer_extension.get_name()
+            # library override mechanisms
+            name = serializer_extension.get_name()
+        elif getattr(getattr(serializer, 'Meta', None), 'ref_name', None) is not None:
+            # local override mechanisms. for compatibility with drf-yasg we support meta ref_name,
+            # though we do not support the serializer inlining feature.
+            # https://drf-yasg.readthedocs.io/en/stable/custom_spec.html#serializer-meta-nested-class
+            name = serializer.Meta.ref_name
+        else:
+            name = serializer.__class__.__name__
+            if name.endswith('Serializer'):
+                name = name[:-10]
 
-        name = serializer.__class__.__name__
-
-        if name.endswith('Serializer'):
-            name = name[:-10]
         if self.method == 'PATCH' and not serializer.read_only and direction == 'request':
             name = 'Patched' + name
 
@@ -825,7 +861,11 @@ class AutoSchema(ViewInspector):
         #   2. concrete component with properties -> use
         #   3. concrete component without properties -> prob. transactional so discard
         #   4. explicit list component -> demultiplexed at usage location so discard
-        if 'oneOf' not in component.schema and not component.schema.get('properties', {}):
+        keep_component = (
+            any(nest_tag in component.schema for nest_tag in ['oneOf', 'allOf', 'anyOf'])
+            or component.schema.get('properties', {})
+        )
+        if not keep_component:
             del self.registry[component]
             return ResolvedComponent(None, None)  # sentinel
         return component
