@@ -7,30 +7,67 @@ from collections.abc import Hashable
 from typing import List, Type, Optional, TypeVar, Union, Generic
 
 import inflection
+import uritemplate
 from django import __version__ as DJANGO_VERSION
+from django.urls.resolvers import _PATH_PARAMETER_COMPONENT_RE, get_resolver
 from django.utils.module_loading import import_string
-from rest_framework import fields, serializers
+from rest_framework import fields, serializers, versioning, exceptions
 
 from drf_spectacular.settings import spectacular_settings
-from drf_spectacular.types import OPENAPI_TYPE_MAPPING, PYTHON_TYPE_MAPPING, OpenApiTypes
+from drf_spectacular.types import (
+    OPENAPI_TYPE_MAPPING, PYTHON_TYPE_MAPPING, DJANGO_PATH_CONVERTER_MAPPING, OpenApiTypes
+)
+from drf_spectacular.utils import OpenApiParameter
 
 T = TypeVar('T')
 
 
+class UnableToProceedError(Exception):
+    pass
+
+
 class GeneratorStats:
-    warn_counter = 0
+    _warn_cache = defaultdict(int)
+    _error_cache = defaultdict(int)
+
+    def __bool__(self):
+        return bool(self._warn_cache or self._error_cache)
+
+    def reset(self):
+        self._warn_cache.clear()
+        self._error_cache.clear()
+
+    def emit(self, msg, severity):
+        assert severity in ['warning', 'error']
+        msg = str(msg)
+        cache = self._warn_cache if severity == 'warning' else self._error_cache
+        if msg not in cache:
+            print(f'{severity.capitalize()} #{len(cache)}: {msg}', file=sys.stderr)
+        cache[msg] += 1
+
+    def emit_summary(self):
+        if self._warn_cache or self._error_cache:
+            print(
+                f'\nSchema generation summary:\n'
+                f'Warnings: {sum(self._warn_cache.values())} ({len(self._warn_cache)} unique)\n'
+                f'Errors:   {sum(self._error_cache.values())} ({len(self._error_cache)} unique)\n',
+                file=sys.stderr
+            )
 
 
 GENERATOR_STATS = GeneratorStats()
 
 
 def warn(msg):
-    GENERATOR_STATS.warn_counter += 1
-    print(f'WARNING #{GENERATOR_STATS.warn_counter}: {msg}', file=sys.stderr)
+    GENERATOR_STATS.emit(msg, 'warning')
+
+
+def error(msg):
+    GENERATOR_STATS.emit(msg, 'error')
 
 
 def reset_generator_stats():
-    GENERATOR_STATS.warn_counter = 0
+    GENERATOR_STATS.reset()
 
 
 def anyisinstance(obj, type_list):
@@ -70,6 +107,20 @@ def is_basic_type(obj):
     return obj in OPENAPI_TYPE_MAPPING or obj in PYTHON_TYPE_MAPPING
 
 
+def has_override(obj, prop):
+    if not hasattr(obj, '_spectacular_annotation'):
+        return False
+    if prop not in obj._spectacular_annotation:
+        return False
+    return True
+
+
+def get_override(obj, prop):
+    if not has_override(obj, prop):
+        return None
+    return obj._spectacular_annotation[prop]
+
+
 def build_basic_type(obj):
     """
     resolve either enum or actual type and yield schema template for modification
@@ -153,6 +204,16 @@ def build_root_object(paths, components):
     return root
 
 
+def safe_ref(schema):
+    """
+    ensure that $ref has its own context and does not remove potential sibling
+    entries when $ref is substituted.
+    """
+    if '$ref' in schema and len(schema) > 1:
+        return {'allOf': [{'$ref': schema.pop('$ref')}], **schema}
+    return schema
+
+
 def get_field_from_model(model, field):
     """
     this is a Django 2.2 compatibility function to access a field through a Deferred Attribute
@@ -174,8 +235,18 @@ def _follow_field_source(model, path):
         else:
             return get_field_from_model(model, field_or_property)
     else:
-        model = field_or_property.field.related_model
-        return follow_field_source(model, path[1:])
+        if isinstance(field_or_property, property):
+            target_model = field_or_property.fget.__annotations__.get('return')
+            if not target_model:
+                raise UnableToProceedError(
+                    f'could not follow field source through intermediate property "{path[0]}" '
+                    f'on model {model}. please add a type hint on the model\'s property to '
+                    f'enable traversal of the source path "{".".join(path)}".'
+                )
+            return _follow_field_source(target_model, path[1:])
+        else:
+            target_model = field_or_property.field.related_model
+            return _follow_field_source(target_model, path[1:])
 
 
 def follow_field_source(model, path):
@@ -187,21 +258,23 @@ def follow_field_source(model, path):
     """
     try:
         return _follow_field_source(model, path)
+    except UnableToProceedError as e:
+        warn(e)
     except:  # noqa: E722
         warn(
             f'could not resolve field on model {model} with path "{".".join(path)}". '
             f'this is likely a custom field that does some unknown magic. maybe '
-            f'consider annotating the field? defaulting to "string".'
+            f'consider annotating the field/property? defaulting to "string".'
         )
 
-        def dummy_property(obj) -> str:
-            pass
-        return dummy_property
+    def dummy_property(obj) -> str:
+        pass
+    return dummy_property
 
 
 def alpha_operation_sorter(endpoint):
     """ sort endpoints first alphanumerically by path, then by method order """
-    path, method, callback = endpoint
+    path, path_regex, method, callback = endpoint
     method_priority = {
         'GET': 0,
         'POST': 1,
@@ -340,14 +413,24 @@ def postprocess_schema_enums(schemas):
     the same choices. Aids client generation to not generate a separate enum for
     every occurrence. only takes effect when replacement is guaranteed to be correct.
     """
+    def iter_prop_containers(schema):
+        if isinstance(schema, list):
+            for item in schema:
+                yield from iter_prop_containers(item)
+        elif isinstance(schema, dict):
+            if schema.get('properties'):
+                yield schema['properties']
+            yield from iter_prop_containers(schema.get('oneOf', []))
+            yield from iter_prop_containers(schema.get('allOf', []))
+
     hash_mapping = defaultdict(set)
     # collect all enums, their names and contents
-    for schema in schemas.values():
-        for prop_name, prop_schema in schema.get('properties', {}).items():
+    for props in iter_prop_containers(list(schemas.values())):
+        for prop_name, prop_schema in props.items():
             if 'enum' not in prop_schema:
                 continue
             hash_mapping[prop_name].add(
-                hash(json.dumps(prop_schema, sort_keys=True))
+                hash(json.dumps(prop_schema['enum'], sort_keys=True))
             )
     # safe replacement requires name to have only one set of enum values
     candidate_enums = {
@@ -355,14 +438,76 @@ def postprocess_schema_enums(schemas):
         if len(prop_hash_set) == 1
     }
     # replace all valid occurrences with enum schema component
-    for schema_name in list(schemas):
-        for prop_name, prop_schema in schemas[schema_name].get('properties', {}).items():
+    for props in iter_prop_containers(list(schemas.values())):
+        for prop_name, prop_schema in props.items():
             if 'enum' not in prop_schema:
                 continue
             if prop_name not in candidate_enums:
                 continue
+
             enum_name = f'{inflection.camelize(prop_name)}Enum'
-            schemas[enum_name] = prop_schema
-            schemas[schema_name]['properties'][prop_name] = {
-                '$ref': f'#/components/schemas/{enum_name}'
+            enum_schema = {k: v for k, v in prop_schema.items() if k in ['type', 'enum']}
+            prop_schema = {k: v for k, v in prop_schema.items() if k not in ['type', 'enum']}
+            prop_schema['$ref'] = f'#/components/schemas/{enum_name}'
+
+            schemas[enum_name] = enum_schema
+            props[prop_name] = safe_ref(prop_schema)
+
+
+def resolve_regex_path_parameter(path_regex, variable):
+    """
+    convert django style path parameters to OpenAPI parameters.
+    TODO also try to handle regular grouped regex parameters
+    """
+    for match in _PATH_PARAMETER_COMPONENT_RE.finditer(path_regex):
+        converter, parameter = match.group('converter'), match.group('parameter')
+
+        if converter and converter.startswith('drf_format_suffix_'):
+            converter = 'drf_format_suffix'  # remove appended options
+
+        if parameter == variable and converter in DJANGO_PATH_CONVERTER_MAPPING:
+            return {
+                'name': parameter,
+                'schema': build_basic_type(DJANGO_PATH_CONVERTER_MAPPING[converter]),
+                'in': OpenApiParameter.PATH,
+                'required': False if converter == 'drf_format_suffix' else True,
             }
+
+    return None
+
+
+def is_versioning_supported(versioning_class):
+    return bool(
+        issubclass(versioning_class, versioning.URLPathVersioning)
+        or issubclass(versioning_class, versioning.NamespaceVersioning)
+    )
+
+
+def operation_matches_version(view, requested_version):
+    try:
+        version, _ = view.determine_version(view.request, **view.kwargs)
+    except exceptions.NotAcceptable:
+        return False
+    else:
+        return str(version) == str(requested_version)
+
+
+def modify_for_versioning(patterns, method, path, view, requested_version):
+    assert view.versioning_class
+
+    from rest_framework.test import APIRequestFactory
+    mocked_request = getattr(APIRequestFactory(), method.lower())(path=path)
+    view.request = mocked_request
+
+    mocked_request.version = requested_version
+
+    if issubclass(view.versioning_class, versioning.URLPathVersioning):
+        version_param = view.versioning_class.version_param
+        # substitute version variable to emulate request
+        path = uritemplate.expand(path, var_dict={version_param: requested_version})
+        # emulate router behaviour by injecting substituted variable into view
+        view.kwargs[version_param] = requested_version
+    elif issubclass(view.versioning_class, versioning.NamespaceVersioning):
+        mocked_request.resolver_match = get_resolver(tuple(patterns)).resolve(path)
+
+    return path
